 
c------------------------------------------------------------------------


c     subroutine for the fast (and approximate) calculation of the 
c     neutral CP-even Higgs-boson masses in the MSSM
c     Two approaches are used:
c     1.) express mlh^2 through a closed formula. Here problems might 
c         arise for small MA
c     2.) the Higgs mass matrix ist determined numerically up to 2-loop,
c         the diagonalisation yields the two Higgs masses

      subroutine mhiggslle(msusytl, msusytr, mtlr, mblr, mt, mtrun, mb, 
     $           mst1, mst2, stt,
     $           ma, tb, alpha, gf, as, cf, el, mz, mw, mue,
     $           mlhlle1, mlhlle2, 
     $           mlhllediag1, mlhllediag2, mhhllediag1, mhhllediag2,
     $           alphadiag1, alphadiag2, mhp)

c     meaning of variables:
c     msusyt[l,r]: SSB terms in the stop mass matrix
c     ms[1,2]: The Susy mass scale: ms1^2 = Msusy^2 + MT^2 (for 1-loop contr.)
c                                   ms2^2 = Msusy^2 + MTrun^2 (2-loop contr.)
c     ms[1,2]2: == ms[1,2]^2
c     mtlr: the off-diagonal entry in the stop mass matrix
c     mt, mb: fermion pole masses
c     mtrun: running top mass, can be used for 2-loop contribution
c     ma, tb: the usual Higgs sector varialbes M_A and tan(beta)
c     gf: Fermi's constant
c     as: strong coupling constant at the scale mt
c     cf = 4/3
c     el: electromagnetic coupling, derived from gf
c     mz, mw: gauge boson masses
c     mlhlle[1,2]: the mass of the light Higgs boson in approach 1
c     m[l,h]hllediag[1,2]: the Higgs masses in approach 2


      double precision msusy, msusytl, msusytr, mtlr, mblr, mt, mtrun, 
     $                 mb, ma, theta,
     $                 gf, as, pi, cf, el, mw, mz, pref, al,
     $                 sw, sw2, sw4, cw2, et, eb, mue,
     $                 ma2, mz2, mw2, mt2, mt4, msfkt,
     $                 msusy2, ms1, ms2, ms12, ms22,
     $                 alpha, tb, b, sb, cb, sb2, cb2, c2b
      double precision MSt1, MSt2, stt
      double precision nc, ng, pt, pb, pf, pg, pgp, p1h, p2h, p2hp
      double precision mlhlle1, mlhlle2, mlhlle1sq, mlhlle2sq, 
     $       mlhllediag1, mlhllediag2, mhhllediag1, mhhllediag2,
     $       mlhllediag1sq, mlhllediag2sq, mhhllediag1sq, mhhllediag2sq,
     $       alphadiag1, alphadiag2, mhp
      double precision m11, m12, m221loop, m222loop
      double precision m11botmix, m12botmix, m22botmix, 
     $                 m11top, m12top, m22top, m22log, 
     $                 m11rest, m12rest, m22rest
      double precision xttilde, vvv, ttt, delmlhsq
      double precision rot(1:2,1:2),rott(1:2,1:2),mhnondiag(1:2,1:2),
     $                 mhdiag(1:2,1:2)
      integer selec2, selec3
      common /selec/selec2, selec3

c      write(*,*) 'LLE parameters:'
c      write(*,*) mt, mtrun
c      write(*,*) mb
c      write(*,*) real(mtlr), real(mblr), real(mue)

      pi = 3.14159265358979d0
      al = el**2/(4d0 * pi)
      cw2 = mw**2/mz**2
      sw2 = 1d0 - mw**2/mz**2
      sw = dsqrt(sw2)
      sw4 = sw2**2
      et = 2d0/3d0
      eb = -1d0/3d0
      b = datan(tb)
      sb = dsin(b)
      sb2 = sb**2
      cb = dcos(b)
      cb2 = cb**2
      c2b = dcos(2d0 * b)
      ma2 = ma**2
      mz2 = mz**2
      mw2 = mw**2
      mt2 = mt**2
      mt4 = mt**4
      if (msusytl.eq.msusytr) then
         ms1 = dsqrt(msusytl**2 + mt**2)
         ms2 = dsqrt(msusytl**2 + mtrun**2)
      else
         ms1 = msfkt(msusytl, msusytr, mt)
         ms2 = msfkt(msusytl, msusytr, mtrun)
      endif
      ms12 = ms1**2
      ms22 = ms2**2
      msusy2 = ms12 - mt**2
      msusy = dsqrt(msusy2)

      nc = 3d0
      ng = 3d0
      pt = nc * (1d0 - 4d0 * et * sw2 + 8d0 * et**2 * sw4)
      pb = nc * (1d0 + 4d0 * eb * sw2 + 8d0 * eb**2 * sw4)
      pf = nc * (ng - 1d0) * (2d0 - 4d0 * sw2 
     $           + 8d0 * (et**2 + eb**2) * sw4) 
     $           + ng * (2d0 - 4d0 * sw2 + 8d0 * sw4)
      pg = -44d0 + 106d0 * sw2 - 62d0 * sw4
      pgp = 10d0 + 34d0 * sw2 - 26d0 * sw4
      p1h = -9d0 * c2b**4 + (1d0 - 2d0 * sw2 + 2d0 * sw4) * c2b**2
      p2h = -10d0 + 2d0 * sw2 - 2d0 * sw4
      p2hp = 8d0 - 22d0 * sw2 + 10d0 * sw4

      theta = 0d0
      if (ma2.ge.mz2) theta = 1d0


c --> 1. approach: the 'direct' formula for mh2

      mlhlle1sq =  
     $    (MA**2 + MZ**2 - Sqrt(MA**4 + MZ**4 - 
     -        2*MA**2*MZ**2*(1 - 8*SB**2 + 8*SB**4)))/2. + 
     -   (al*MT**4*((-3*MT**4*MtLR**8)/(55.99999999999999*Ms1**12) + 
     n    (1d0/4d0 * MZ**2/MT**2 - 11d0/80d0 * MZ**4/MT**4)
     -       + (MtLR**2*(1.5 - (3*MT**2)/(4.*Ms1**2) + 
     $                       0d0 - 
     -             MZ**2/(2.*MT**2)))/Ms1**2 + 
     -     (MtLR**6*((-3*MT**2)/(40.*Ms1**2) + (3*MT**4)/(10.*Ms1**4) - 
     -                                    0d0  ))/Ms1**6 + 
     -    (MtLR**4*(-0.125 + MT**2/(2.*Ms1**2) - (3*MT**4)/(8.*Ms1**4) - 
     -                                0d0 + 
     $                             0d0 ))/Ms1**4
     -        )*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4))))/
     -    (2.*MW**2*Pi*SB**2*sw2)  

      mlhlle1sq = mlhlle1sq -
     -   (al*MT**4*(1.5 - (3*MZ**2*SB**2)/(4.*MT**2) + 
     -        ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*
     $     SB**2)/(72.*MT**4) + 
     -        (4*(MA**2 + MZ**2)*SB**2*(1 - SB**2)*
     -           ((3*MZ**2)/(8.*MT**2) - 
     -             ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*SB**2)/
     $     (72.*MT**4))
     -           )/Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4))
     -          + ((-MA**2 + MZ**2)*(1 - 2*SB**2)*
     -           (1.5 - (3*MZ**2*SB**2)/(4.*MT**2) - 
     -             ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*SB**2*
     -                (1 - 2*SB**2))/(72.*MT**4)))/
     -     Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4)))*
     -      Log(MT**2/Ms1**2))/(2.*MW**2*Pi*SB**2*sw2)

      mlhlle1sq = mlhlle1sq +
     -   (al*MZ**4*(Log(Msusy**2/MZ**2)*
     -         ((36*MB**4)/MZ**4 - (18*MB**2*Cos(2*b))/MZ**2 + 
     -           (3*(1 - (4*sw2)/3. + (8*sw4)/9.) + 
     -              6*(2 - 4*sw2 + (40*sw4)/9.) + 
     -              3*(2 - 4*sw2 + 8*sw4))*Cos(2*b)**2 - 
     -           2*(18 + 12*sw2 - 16*sw4)*Cos(b)**2*Sin(b)**2 + 
     -           (-54 + 108*sw2 - 64*sw4)*
     $     (Cos(b)**4 + Sin(b)**4)) - 
     -        theta * Log(MA**2/MZ**2)*(-((1 - 2*sw2 + 2*sw4)*
     $     Cos(2*b)**2) + 
     -           9*Cos(2*b)**4 - 
     -           2*(8 - 22*sw2 + 10*sw4)*Cos(b)**2*Sin(b)**2 + 
     -           (-10 + 2*sw2 - 2*sw4)*
     $     (Cos(b)**4 + Sin(b)**4))))/
     -    (23.99999999999999*MW**2*Pi*sw2)

      mlhlle1sq = mlhlle1sq +
     -   (3*al*((2*MB**4*MbLR**2*(1 - MbLR**2/
     $     (11.99999999999999*Msusy**2)))/
     -         Msusy**2 - (MB**2*MZ**2*Cos(2*b)*
     -           (MbLR**2 + (-(Mue**2*Tan(b)**2) + 
     $     (MbLR + Mue*Tan(b))**2)/3.)
     -           )/(2.*Msusy**2)))/(4.*MW**2*Pi*sw2)
      if (mlhlle1sq.gt.0d0) then
         mlhlle1 = dsqrt(mlhlle1sq)
      else
         mlhlle1 = 119.9999d0
      endif


c      write(*,*) 'lle 2loop'
      if (selec2.ge.3) then
c         write(*,*) '1-loop with mtrun => changed 2-loop formula'
      mlhlle2sq = (
     $     mlhlle1sq
     -   + (al*as*MT**4*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4)))*
     -      ((6*MtLR)/Ms2 - (12*MtLR**4)/(17.*Ms2**4) + 
     -        (+2d0 - (3*MtLR**2)/Ms2**2)*Log(Ms2**2/MT**2) - 
     -        3*Log(Ms2**2/MT**2)**2
     $      - 4d0 + 8d0 * MtLR**2/Ms2**2))/
     $     (2.*MW**2*Pi**2*SB**2*sw2) 
     $     )
      else
c         write(*,*) '1-loop with mt pole => original 2-loop formula'
      mlhlle2sq = (
     $     mlhlle1sq
     -   + (al*as*MT**4*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4)))*
     -      ((6*MtLR)/Ms1 - (3*MtLR**4)/(4.*Ms1**4) + 
     -        (-6 - (3*MtLR**2)/Ms1**2)*Log(Ms1**2/MT**2) - 
     -        3*Log(Ms1**2/MT**2)**2))/
     $     (2.*MW**2*Pi**2*SB**2*sw2) 
     $     )
      endif

      if (mlhlle2sq.gt.0d0) then
         mlhlle2 = dsqrt(mlhlle2sq)
      else
         mlhlle2 = 119.9999d0
      endif


c ----------------------------------------------------------------------

c --> 2. approach: the Higgs mass matrix


c      write(*,*) '2. approach'
      pref = el**2 * mz2 / (96d0 * Pi**2 * cw2 * sw2)
      mwmzfak = 32d0 * mw**4 - 40d0 * mw2 * mz2 + 17d0 * mz**4

c      write(*,*) '2. approach'
c --> the top sector at 1-loop
      m11top = -el**2/(288d0 * mw2 * pi**2 * sw2) 
     $         * mwmzfak * cb2 * dlog(mt2/ms12)
      m12top = el**2 * cb/(288d0 * sb * mw2 * pi**2 * sw2) 
     $         * (- 27 * mt2 * mz2 + mwmzfak * sb2) * dlog(mt2/ms12)
      m22top = el**2/(288d0 * sb2 * mw2 * pi**2 * sw2) *
     $  ((-108d0 * mt**4 + 54d0 * mt2 * mz2 * sb2 - mwmzfak * sb**4) *
     $     dlog(mt2/ms12)
     n     + (2d0 * MZ**2/MT**2 - 11d0/10d0 * MZ**4/MT**4) * 9d0 * MT**4
     $     +9d0 * (MtLR/Ms1)**2 * (MT/Ms1)**2 *
     $ (-6d0 * mt**4                   + 4d0 * ms12 * (3d0 * mt2 - mz2))
     $     +9d0/10d0 * (MtLR/Ms1)**4 * (MT/Ms1)**4 *
     $     (-10d0 * ms1**4 + 4 * ms1**2 * (10d0 * mt2            ) +
     $     15d0 * mt2 * (-2d0 * mt2      ))
     $     +9d0/35d0 * (MtLR/Ms1)**6 * (MT/Ms1)**6 *
     $     (-21d0 * ms1**4 + 12d0 * ms1**2 * (7d0 * mt2            ) +
     $     35d0 * mt2 * (-2d0 * mt2      ))
     $     -288d0/469762048d0 * (MtLR/Ms1)**8 * (MT/Ms1)**8 *
     $     (6291456d0 * ms1**4 + ms1**2 * (-25165824d0 * mt2
     $                      ) + 11010048d0 * (2d0 * mt**4            )))
      

c      write(*,*) '2. approach'
c --> all other sectors except for the sbottom mixing
      m11rest = pref * cb2 *
     $          ((  12d0 * nc * mb**4/(mz**4 * cb**4) 
     $           - 6d0 * nc * mb**2/(mz**2 * cb**2) 
     $           + pb + pf + pg + p2h ) * dlog(msusy2/mz2)
     $          + theta * (p1h - p2h) * dlog(ma2/mz2) )
      m12rest = -pref * sb * cb *
     $          ((-3d0 * nc * mb**2/(mz2 * cb2) 
     $            + pb + pf + pgp + p2hp) * dlog(msusy2/mz2) 
     $           - theta * (p1h + p2hp) * dlog(ma2/mz2) )
      m22rest = pref * sb2 * 
     $          ((pb + pf + pg + p2h) * dlog(msusy2/mz2) 
     $           + theta * (p1h - p2h) * dlog(ma2/mz2) )
       
        
c      write(*,*) '2. approach'
c --> the bottom mixing at 1-loop 
      m11botmix = 
     &  (AL*MW**(-2)*NC/SW2*(-(MB**2*MSUSY**(-2)*MZ**2*(MTLR+
     &  MUE/tan(B))*(MBLR+((MTLR+MUE/tan(B)))/3D0))+4D0*CB**(-2)*
     &  MB**4*MBLR*MSUSY**(-2)*(MTLR+MUE/tan(b))*(1D0-(MBLR*MSUSY**
     &  (-2)*(MTLR+MUE/tan(b)))/12D0)))/(8D0*PI)
      m12botmix =
     &  -((AL*MW**(-2)*NC/SW2*(4D0*CB**(-2)*MB**4*MBLR*MSUSY**
     &  (-2)*MUE*(1D0-(MBLR*MSUSY**(-2)*(MTLR+MUE/tan(b)))/6D0)-MB**
     &  2*MSUSY**(-2)*MZ**2*(MBLR*(MTLR+2D0*MUE/tan(b))+((MUE**2+
     &  (MTLR+MUE/tan(b))**2))/3D0)*TAN(B)))/(16D0*PI))
      m22botmix = 
     &  (AL*MW**(-2)*NC/SW2*(-((1D0*CB**(-2)*MB**4*MBLR**2*
     &  MSUSY**(-4)*MUE**2)/3D0)-MB**2*MSUSY**(-2)*MUE*MZ**2*TAN(B)*
     &  (MBLR+(MUE*TAN(B))/3D0)))/(8D0*PI)


c --> the top sector at 2-loop
      if (selec2.ge.3) then
c         write(*,*) '1-loop with mtrun => changed 2-loop formula'
         m22log = (al*as*MTrun**4/dsin(b)**2 * 
     -      ((6*MtLR)/Ms2 - (12*MtLR**4)/(17.*Ms2**4) + 
     -        (+2d0 - (3*MtLR**2)/Ms2**2)*dLog(Ms2**2/MT**2) - 
     -        3*dLog(Ms2**2/MT**2)**2
     $      - 4d0 + 8d0 * MtLR**2/Ms2**2))/
     $        (MW**2*Pi**2*SW2)
      else
c         write(*,*) '1-loop with mt pole => original 2-loop formula'
         m22log = (al*as*MT**4/dsin(b)**2 * 
     $        ((6*MtLR)/Ms2 - (3*MtLR**4)/(4.*Ms2**4) + 
     $        (-6 - (3*MtLR**2)/Ms2**2)*dLog(Ms2**2/MT**2) 
     $        - 3*dLog(Ms2**2/MT**2)**2
     $        ))/(MW**2*Pi**2*SW2)
      endif

c$$$      write(*,*) 'Parameters in LLESub:'
c$$$      write(*,*) 'Msusy, MT, MtLR,'
c$$$      write(*,*) 'M22top1loop, M22rest, M22botmix, M22top2loop'
c$$$      write(*,*) real(msusy), real(mt), real(mtlr)
c$$$      write(*,*) real(m22top), real(m22rest), real(m22botmix), 
c$$$     $           real(m22log)
c$$$      write(*,*) cf, el, al, sw, pi, sb, mtrun, mw, llexpansionp2mtrun

c     including the leading Yukawa term for the light Higgs mass
c     this term is taken from Carena, Espinosa, Quiros, Wagner
c     Nucl. Phys. B461 (1996) 407

      write(*,*) 'adding Yukawa term'

      xttilde = (((MSt2**2 - MSt1**2)/(4d0*mt**2) * 
     $            (2d0 * stt * dsqrt(1d0 - stt**2))**2 )**2 *
     $           (2d0 - (MSt2**2 + MSt1**2)/(MSt2**2 - MSt1**2) *
     $                  dlog(MSt2**2/MSt1**2)) +
     $           (MSt2**2 - MSt1**2)/(2d0 * mt**2) *
     $            (2d0 * stt * dsqrt(1d0 - stt**2))**2 *
     $                  dlog(MSt2**2/MSt1**2) )
      ttt = .5d0 * dlog(MSt2**2 * MSt1**2/mt**4)
      vvv = 174.1d0
      delmlhsq = ( 3d0/(4d0 * pi**2) * mt**4/vvv**2 *
     $            ( 1d0/(16d0 * pi**2) * 3d0/2d0 * mt**2/vvv**2 *
     $             ( xttilde * ttt + ttt**2)))


c----------------------------------------------------------------


      m11 = m11top + m11rest + m11botmix
      m12 = m12top + m12rest + m12botmix
      m221loop = m22top + m22rest + m22botmix
      m222loop = m221loop + m22log
      if ((selec2.eq.2).or.(selec2.eq.4)) then
         m222loop = m222loop + delmlhsq/sb2
      endif


c----------------------------------------------------------------


      mlhllediag1sq = 
     $     (ma2 + mz2 + m221loop + m11 -
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m221loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m221loop * cb2 
     $                     + m221loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m221loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      mlhllediag2sq = 
     $     (ma2 + mz2 + m222loop + m11 -
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m222loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m222loop * cb2 
     $                     + m222loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m222loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      mhhllediag1sq = 
     $     (ma2 + mz2 + m221loop + m11 +
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m221loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m221loop * cb2 
     $                     + m221loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m221loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      mhhllediag2sq = 
     $     (ma2 + mz2 + m222loop + m11 +
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m222loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m222loop * cb2 
     $                     + m222loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m222loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0


      if (mlhllediag1sq.gt.0d0) then
         mlhllediag1 = dsqrt(mlhllediag1sq)
         alphadiag1 = datan((-(ma2 + mz2) * sb * cb + m12)/
     $                      (mz2 * cb2 + ma2 * sb2 + m11 
     $                       - mlhllediag1sq))
      else
         mlhllediag1 = 119.9999d0
         alphadiag1 = 0d0
      endif

      if (mlhllediag2sq.gt.0d0) then
         mlhllediag2 = dsqrt(mlhllediag2sq)
         alphadiag2 = datan((-(ma2 + mz2) * sb * cb + m12)/
     $                      (mz2 * cb2 + ma2 * sb2 + m11 
     $                       - mlhllediag2sq))
      else
         mlhllediag2 = 119.9999d0
         alphadiag2 = 0d0
      endif

      mhhllediag1 = dsqrt(mhhllediag1sq)
      mhhllediag2 = dsqrt(mhhllediag2sq)


c----------------------------------------------------------------

      if ((selec2.eq.2).or.(selec2.eq.4)) then
c     including the leading Yukawa term for the light Higgs mass
c     this term is taken from Carena, Espinosa, Quiros, Wagner
c     Nucl. Phys. B461 (1996) 407

      mlhlle2 = dsqrt(mlhlle2**2 + delmlhsq)
c      write(*,*) real(mlhlle2), real(mlhllediag2)

      endif

c----------------------------------------------------------------

c --> rotation check
      rot(1,1) = dcos(alpha)
      rot(1,2) = -dsin(alpha)
      rot(2,1) = -rot(1,2)
      rot(2,2) = rot(1,1)
      rott(1,1) = rot(1,1)
      rott(1,2) = rot(2,1)
      rott(2,1) = rot(1,2)
      rott(2,2) = rot(2,2)
      mhnondiag(1,1) = ma2 * sb2 + mz2 * cb2
      mhnondiag(1,2) = - (ma2 + mz2) * sb * cb
      mhnondiag(2,1) = mhnondiag(1,2)
      mhnondiag(2,2) = ma2 * cb2 + mz2 * sb2
      call diagonalization(rott, mhnondiag, rot, mhdiag)
      if (dabs(mhdiag(1,2)).le.1d-6) mhdiag(1,2) = 0d0
      if (dabs(mhdiag(2,1)).le.1d-6) mhdiag(2,1) = 0d0
c      write(*,*) 'tree level diagonalization:'
c      write(*,*) real(dsqrt(mhdiag(1,1))), real(mhdiag(1,2))
c      write(*,*) real(mhdiag(2,1)), real(dsqrt(mhdiag(2,2)))

      rot(1,1) = dcos(alphadiag1)
      rot(1,2) = -dsin(alphadiag1)
      rot(2,1) = -rot(1,2)
      rot(2,2) = rot(1,1)
      rott(1,1) = rot(1,1)
      rott(1,2) = rot(2,1)
      rott(2,1) = rot(1,2)
      rott(2,2) = rot(2,2)
      mhnondiag(1,1) = mhnondiag(1,1) + m11
      mhnondiag(1,2) = mhnondiag(1,2) + m12
      mhnondiag(2,1) = mhnondiag(1,2)
      mhnondiag(2,2) = mhnondiag(2,2) + m221loop
      call diagonalization(rott, mhnondiag, rot, mhdiag)
      if (dabs(mhdiag(1,2)).le.1d-6) mhdiag(1,2) = 0d0
      if (dabs(mhdiag(2,1)).le.1d-6) mhdiag(2,1) = 0d0
c      write(*,*) '1-loop diagonalization:'
c      write(*,*) real(dsqrt(mhdiag(1,1))), real(mhdiag(1,2))
c      write(*,*) real(mhdiag(2,1)), real(dsqrt(mhdiag(2,2)))
      if ((dabs(mlhllediag1sq - mhdiag(2,2)).gt.1d-4).or.
     $    (dabs(mhhllediag1sq - mhdiag(1,1)).gt.1d-4).or.
     $    (dabs(mhdiag(1,2)).gt.1d-6).or.
     $    (dabs(mhdiag(2,1)).gt.1d-6)) then
         write(*,*) "WARNING --- WARNING --- WARNING --- WARNING"
         write(*,*) "Error in 1-loop rotation !!!!!!!!!!!!!!!!!!"
         write(*,*) "WARNING --- WARNING --- WARNING --- WARNING"
      endif

      rot(1,1) = dcos(alphadiag2)
      rot(1,2) = -dsin(alphadiag2)
      rot(2,1) = -rot(1,2)
      rot(2,2) = rot(1,1)
      rott(1,1) = rot(1,1)
      rott(1,2) = rot(2,1)
      rott(2,1) = rot(1,2)
      rott(2,2) = rot(2,2)
      mhnondiag(1,1) = mhnondiag(1,1)
      mhnondiag(1,2) = mhnondiag(1,2)
      mhnondiag(2,1) = mhnondiag(1,2)
      mhnondiag(2,2) = mhnondiag(2,2) - m221loop + m222loop
      call diagonalization(rott, mhnondiag, rot, mhdiag)
      if (dabs(mhdiag(1,2)).le.1d-6) mhdiag(1,2) = 0d0
      if (dabs(mhdiag(2,1)).le.1d-6) mhdiag(2,1) = 0d0
c      write(*,*) '2-loop diagonalization:'
c      write(*,*) real(dsqrt(mhdiag(1,1))), real(mhdiag(1,2))
c      write(*,*) real(mhdiag(2,1)), real(dsqrt(mhdiag(2,2)))
      if ((dabs(mlhllediag2sq - mhdiag(2,2)).gt.1d-4).or.
     $    (dabs(mhhllediag2sq - mhdiag(1,1)).gt.1d-4).or.
     $    (dabs(mhdiag(1,2)).gt.1d-6).or.
     $    (dabs(mhdiag(2,1)).gt.1d-6)) then
         write(*,*) "WARNING --- WARNING --- WARNING --- WARNING"
         write(*,*) "Error in 2-loop rotation !!!!!!!!!!!!!!!!!!"
         write(*,*) "WARNING --- WARNING --- WARNING --- WARNING"
      endif

c --> end of rotation check

c-------------------------------------------------------------------
c --> charged Higgs at one loop
      mhp = dsqrt(ma2 + mw2 + 3d0/(4d0 * pi) * al/(sw2 * mw2) *
     $            (  2d0 * mt2 * mb**2/(sb2 * cb2) 
     $             - mw2 * (mt2/sb2 + mb**2/cb2) + 2d0/3d0 * mw2**2)
     $            * dlog(msusy/mt)
     $            + mw2/(6d0 * pi) * 15d0 * al/cw2 * dlog(msusy/mw))


      end
